@@ -248,6 +248,7 @@ sub graph_startup {
     # NOTE!  Some of these options are available in graph_main too
     # if you make changes here, make them there too.
 
+    my $debug;
     &print_usage_and_exit
         unless GetOptions (
                 "force!"        => \$force_graphing,
@@ -280,7 +281,8 @@ sub graph_startup {
                 "cron!"         => \$cron,
                 "fork!"         => \$do_fork,
                 "n=n"           => \$max_running,
-                "help"          => \$do_usage
+                "help"          => \$do_usage,
+                "debug!"        => \$debug,
         );
 
     if ($do_version) {
@@ -304,6 +306,8 @@ sub graph_startup {
 	$config = munin_readconfig_part('datafile', 0);
     }
 
+    $config->{debug} = $debug;
+
     my $palette = &munin_get($config, "palette", "default");
 
     $max_running = &munin_get($config, "max_graph_jobs", $max_running);
@@ -405,16 +409,10 @@ sub graph_main {
         autoflush $STATS 1;
     }
 
-    INFO "Starting munin-graph";
-
     process_work(@limit_hosts);
 
     $graph_time = sprintf("%.2f", (Time::HiRes::time - $graph_time));
 
-    INFO "Munin-graph finished ($graph_time sec)";
-
-    print $STATS "GT|total|$graph_time\n" unless $skip_stats;
-
     rename(
         "$config->{dbdir}/munin-graph.stats.tmp",
         "$config->{dbdir}/munin-graph.stats"
@@ -1044,6 +1042,10 @@ sub process_service {
             and graph_by_minute($service)) {
             push(@rrd, expand_cdef($service, \$rrdname, "$fname,60,*"));
         }
+        if (    munin_get($field, "type", "GAUGE") ne "GAUGE"
+            and graph_by_hour($service)) {
+            push(@rrd, expand_cdef($service, \$rrdname, "$fname,3600,*"));
+        }
 
         if (my $tmpcdef = munin_get($field, "cdef")) {
             push(@rrd, expand_cdef($service, \$rrdname, $tmpcdef));
@@ -1453,7 +1455,7 @@ sub process_service {
     if (munin_get_bool($service, "graph_sums", 0)) {
         foreach my $time (keys %sumtimes) {
             my $picfilename = get_picture_filename($service, $time, 1);
-	    INFO "Looking into drawing $picfilename";
+	    DEBUG "Looking into drawing $picfilename";
             (my $picdirname = $picfilename) =~ s/\/[^\/]+$//;
             next unless ($draw{"sum" . $time});
             my @rrd_sum;
@@ -1497,6 +1499,10 @@ sub process_service {
                                 # Already multiplied by 60
                                 push @replace,
                                     "CDEF:x$fname=PREV($fname),UN,0,PREV($fname),IF,$fname,+,5,*,6,*";
+                            } elsif (graph_by_hour($service)) {
+                                # Already multiplied by 3600, have to *divide* by 12
+                                push @replace,
+                                    "CDEF:x$fname=PREV($fname),UN,0,PREV($fname),IF,$fname,+,12,/,6,*";
                             }
                             else {
                                 push @replace,
@@ -1510,6 +1516,9 @@ sub process_service {
 
                                 # Already multiplied by 60
                                 push @replace, "CDEF:x$fname=$fname,5,*,288,*";
+                            } elsif (graph_by_hour($service)) {
+                                # Already multiplied by 3600, have to *divide* by 12
+                                push @replace, "CDEF:x$fname=$fname,12,/,288,*";
                             }
                             else {
                                 push @replace,
@@ -1558,7 +1567,7 @@ sub process_service {
     } # if graph_sums
 
     $service_time = sprintf("%.2f", (Time::HiRes::time - $service_time));
-    INFO "[INFO] Graphed service $skeypath ($service_time sec for $nb_graphs_drawn graphs)";
+    DEBUG "[DEBUG] Graphed service $skeypath ($service_time sec for $nb_graphs_drawn graphs)";
     print $STATS "GS|$service_time\n" unless $skip_stats;
 
     foreach (@added) {
@@ -1604,6 +1613,8 @@ sub handle_trends {
             $colour = "$service->{$fieldname}{'colour'}66";
         }                                                                                                                                                    
                                                                                                                                                              
+        my $rrd_fieldname = get_field_name($fieldname);
+
         my $cdef = "";
         if (defined $service->{$fieldname}{'cdef'}) {
             $cdef = "cdef";
@@ -1611,8 +1622,8 @@ sub handle_trends {
 
         #trends
         if (defined $service->{$fieldname}{'trend'} and $service->{$fieldname}{'trend'} eq 'yes') {
-            push (@complete, "CDEF:t$fieldname=c$cdef$fieldname,$futuretime,TRENDNAN");
-            push (@complete, "LINE1:t$fieldname#$colour:$fieldname trend\\l");
+            push (@complete, "CDEF:t$rrd_fieldname=c$cdef$rrd_fieldname,$futuretime,TRENDNAN");
+            push (@complete, "LINE1:t$rrd_fieldname#$colour:$service->{$fieldname}->{'label'} trend\\l");
             DEBUG "[DEBUG] set trend for $fieldname\n";
         }
 
@@ -1622,8 +1633,8 @@ sub handle_trends {
             my @predict = split(",", $service->{$fieldname}{'predict'});
             my $predictiontime = int ($futuretime / $predict[0]) + 2; #2 needed for 1 day
             my $smooth = $predict[1]*$resolutions{$time};
-            push (@complete, "CDEF:p$fieldname=$predict[0],-$predictiontime,$smooth,c$cdef$fieldname,PREDICT");
-            push (@complete, "LINE1:p$fieldname#$colour:$fieldname prediction\\l");
+            push (@complete, "CDEF:p$rrd_fieldname=$predict[0],-$predictiontime,$smooth,c$cdef$rrd_fieldname,PREDICT");
+            push (@complete, "LINE1:p$rrd_fieldname#$colour:$service->{$fieldname}->{'label'} prediction\\l");
             DEBUG "[DEBUG] set prediction for $fieldname\n";
         }
     }
@@ -1636,7 +1647,6 @@ sub handle_trends {
 
     # If pinpointing, --end should *NOT* be changed
     if (! $pinpoint) {
-        if (time - 300 < $lastupdate) {
             if (@added) { # stop one period earlier if it's a .sum or .stack
                 push @complete, "--end",
                     (int(($enddate-$resolutions{$time}) / $resolutions{$time})) * $resolutions{$time};
@@ -1644,7 +1654,6 @@ sub handle_trends {
                 push @complete, "--end",
                     (int($enddate / $resolutions{$time})) * $resolutions{$time};
             }
-        }
     }
 
     return @complete;
@@ -1697,6 +1706,12 @@ sub graph_by_minute {
     return (munin_get($service, "graph_period", "second") eq "minute");
 }
 
+sub graph_by_hour {
+    my $service = shift;
+
+    return (munin_get($service, "graph_period", "second") eq "hour");
+}
+
 sub orig_to_cdef {
     my $service   = shift;
     my $fieldname = shift;
